==> angr-9.0.4885.tar.gz: angr-9.0.4885/angr/procedures/linux_kernel/arm_user_helpers.py <==
import angr

"""
references:
https://elixir.bootlin.com/linux/latest/source/arch/arm/kernel/entry-armv.S
https://www.kernel.org/doc/Documentation/arm/kernel_user_helpers.txt
"""

class _kuser_helper_version(angr.SimProcedure):
    # pylint: disable=arguments-differ
    kuser_addr = 0xffff0ffc
    def run(self):
        # hardcoded version number extracted from QEMU
        self.state.regs.r0 = 0x884c0
        return

class _kuser_helper_get_tls(angr.SimProcedure):
    # pylint: disable=arguments-differ
    kuser_addr = 0xffff0fe0
    def run(self):
        self.state.regs.r0 = self.project.loader.tls.threads[0].user_thread_pointer
        return

class _kuser_cmpxchg(angr.SimProcedure):
    # pylint: disable=arguments-differ
    kuser_addr = 0xffff0fc0
    def run(self, oldval, newval, ptr):
        ptrval = self.state.memory.load(ptr, self.project.arch.bytes)
        retval = ptrval - oldval

        # the return value can't be symbolic
        if self.state.solver.symbolic(retval):
            raise angr.errors.AngrValueError("_kuser_cmpxchg returns symbolic value")

        # handle cmpxchg
        if self.state.solver.is_true(retval == 0):
            self.state.memory.store(ptr, newval)
            # set CARRY flag
            self.state.regs.flags |= 0x20000000
        else:
            # zero CARRY flag
            self.state.regs.flags &= 0xdfffffff
        return retval

class _kuser_memory_barrier(angr.SimProcedure):
    # pylint: disable=arguments-differ
    kuser_addr = 0xffff0fa0
    def run(self):
        raise NotImplementedError(f"{self.__class__.__name__} is not implemented")

class _kuser_cmpxchg64(angr.SimProcedure):
    # pylint: disable=arguments-differ
    kuser_addr = 0xffff0f60
    def run(self):
        raise NotImplementedError(f"{self.__class__.__name__} is not implemented")

==> angr-9.0.5002.tar.gz: angr-9.0.5002/angr/procedures/linux_kernel/arm_user_helpers.py <==
import angr

"""
references:
https://elixir.bootlin.com/linux/latest/source/arch/arm/kernel/entry-armv.S
https://www.kernel.org/doc/Documentation/arm/kernel_user_helpers.txt
"""

class _kuser_helper_version(angr.SimProcedure):
    # pylint: disable=arguments-differ
    kuser_addr = 0xffff0ffc
    def run(self):
        # hardcoded version number extracted from QEMU
        self.state.regs.r0 = 0x884c0
        return

class _kuser_helper_get_tls(angr.SimProcedure):
    # pylint: disable=arguments-differ
    kuser_addr = 0xffff0fe0
    def run(self):
        self.state.regs.r0 = self.project.loader.tls.threads[0].user_thread_pointer
        return

class _kuser_cmpxchg(angr.SimProcedure):
    # pylint: disable=arguments-differ
    kuser_addr = 0xffff0fc0
    def run(self, oldval, newval, ptr):
        currentval = self.state.mem[ptr].dword.resolved
        success = currentval == oldval
        self.state.memory.store(ptr, newval, condition=success)

        # set flags
        retval = currentval - oldval
        self.state.regs.cc_op = 2
        self.state.regs.cc_dep1 = 0
        self.state.regs.cc_dep2 = retval
        self.state.regs.cc_ndep = 0
        return retval

class _kuser_memory_barrier(angr.SimProcedure):
    # pylint: disable=arguments-differ
    kuser_addr = 0xffff0fa0
    def run(self):
        raise NotImplementedError(f"{self.__class__.__name__} is not implemented")

class _kuser_cmpxchg64(angr.SimProcedure):
    # pylint: disable=arguments-differ
    kuser_addr = 0xffff0f60
    def run(self):
        raise NotImplementedError(f"{self.__class__.__name__} is not implemented")
